{% load i18n %}
<label for="{{ widget.attrs.id }}_select_all">
    <input type="checkbox" id="{{ widget.attrs.id }}_select_all">
    {% trans "Select all"%}
</label>
{% include "django/forms/widgets/multiple_input.html" %}
